@@ -15,6 +15,8 @@ core.features = {
 	httpfetch_binary_data = true,
 	formspec_version_element = true,
 	area_store_persistent_ids = true,
+	pathfinder_works = true,
+	object_step_has_moveresult = true,
 }
 
 function core.has_feature(arg)
