@@ -125,6 +125,12 @@ private:
 	u8 m_last_light = 255;
 	bool m_is_visible = false;
 	s8 m_glow = 0;
+	// Material
+	video::E_MATERIAL_TYPE m_material_type;
+	// Settings
+	bool m_enable_shaders = false;
+
+	bool visualExpiryRequired(const ObjectProperties &newprops) const;
 
 public:
 	GenericCAO(Client *client, ClientEnvironment *env);
@@ -165,9 +171,9 @@ public:
 
 	const bool isImmortal();
 
-	scene::ISceneNode *getSceneNode();
+	scene::ISceneNode *getSceneNode() const;
 
-	scene::IAnimatedMeshSceneNode *getAnimatedMeshSceneNode();
+	scene::IAnimatedMeshSceneNode *getAnimatedMeshSceneNode() const;
 
 	// m_matrixnode controls the position and rotation of the child node
 	// for all scene nodes, as a workaround for an Irrlicht problem with
@@ -182,10 +188,11 @@ public:
 		return m_matrixnode->getRelativeTransformationMatrix();
 	}
 
-	inline const core::matrix4 &getAbsolutePosRotMatrix() const
+	inline const core::matrix4 *getAbsolutePosRotMatrix() const
 	{
-		assert(m_matrixnode);
-		return m_matrixnode->getAbsoluteTransformation();
+		if (!m_matrixnode)
+			return nullptr;
+		return &m_matrixnode->getAbsoluteTransformation();
 	}
 
 	inline f32 getStepHeight() const
@@ -230,11 +237,16 @@ public:
 		m_visuals_expired = true;
 	}
 
-	void updateLight(u8 light_at_pos);
+	void updateLight(u32 day_night_ratio);
 
-	void updateLightNoCheck(u8 light_at_pos);
+	void setNodeLight(u8 light);
 
-	v3s16 getLightPosition();
+	/* Get light position(s).
+	 * returns number of positions written into pos[], which must have space
+	 * for at least 3 vectors. */
+	u16 getLightPosition(v3s16 *pos);
+
+	void updateNametag();
 
 	void updateNodePos();
 
