@@ -171,7 +171,8 @@ void draw2DImageFilterScaled(video::IVideoDriver *driver, video::ITexture *txr,
 }
 
 void draw2DImage9Slice(video::IVideoDriver *driver, video::ITexture *texture,
-		const core::rect<s32> &rect, const core::rect<s32> &middle)
+		const core::rect<s32> &rect, const core::rect<s32> &middle,
+		const core::rect<s32> *cliprect)
 {
 	const video::SColor color(255,255,255,255);
 	const video::SColor colors[] = {color,color,color,color};
@@ -222,9 +223,7 @@ void draw2DImage9Slice(video::IVideoDriver *driver, video::ITexture *texture,
 				break;
 			}
 
-			draw2DImageFilterScaled(driver, texture, dest,
-					src,
-					NULL/*&AbsoluteClippingRect*/, colors, true);
+			draw2DImageFilterScaled(driver, texture, dest, src, cliprect, colors, true);
 		}
 	}
 }
