@@ -150,6 +150,8 @@ enum MaterialType{
 	TILE_MATERIAL_WAVING_LIQUID_BASIC,
 	TILE_MATERIAL_WAVING_LIQUID_TRANSPARENT,
 	TILE_MATERIAL_WAVING_LIQUID_OPAQUE,
+	TILE_MATERIAL_PLAIN,
+	TILE_MATERIAL_PLAIN_ALPHA
 };
 
 // Material flags
