@@ -339,12 +339,18 @@ public:
 	u8 getMinor() const { return m_version_minor; }
 	u8 getPatch() const { return m_version_patch; }
 	const std::string &getFull() const { return m_full_version; }
+	
+	void setLangCode(const std::string &code) { m_lang_code = code; }
+	const std::string &getLangCode() const { return m_lang_code; }
 private:
 	// Version is stored in here after INIT before INIT2
 	u8 m_pending_serialization_version = SER_FMT_VER_INVALID;
 
 	/* current state of client */
 	ClientState m_state = CS_Created;
+	
+	// Client sent language code
+	std::string m_lang_code;
 
 	/*
 		Blocks that have been sent to client.
