@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef EXCEPTIONS_HEADER
-#define EXCEPTIONS_HEADER
+#pragma once
 
 #include <exception>
 #include <string>
@@ -28,7 +27,8 @@ class BaseException : public std::exception
 {
 public:
 	BaseException(const std::string &s) throw(): m_s(s) {}
-	~BaseException() throw() {}
+	~BaseException() throw() = default;
+
 	virtual const char * what() const throw()
 	{
 		return m_s.c_str();
@@ -37,16 +37,6 @@ protected:
 	std::string m_s;
 };
 
-class AsyncQueuedException : public BaseException {
-public:
-	AsyncQueuedException(const std::string &s): BaseException(s) {}
-};
-
-class NotImplementedException : public BaseException {
-public:
-	NotImplementedException(const std::string &s): BaseException(s) {}
-};
-
 class AlreadyExistsException : public BaseException {
 public:
 	AlreadyExistsException(const std::string &s): BaseException(s) {}
@@ -77,16 +67,6 @@ public:
 	PacketError(const std::string &s): BaseException(s) {}
 };
 
-class LoadError : public BaseException {
-public:
-	LoadError(const std::string &s): BaseException(s) {}
-};
-
-class ContainerFullException : public BaseException {
-public:
-	ContainerFullException(const std::string &s): BaseException(s) {}
-};
-
 class SettingNotFoundException : public BaseException {
 public:
 	SettingNotFoundException(const std::string &s): BaseException(s) {}
@@ -97,16 +77,6 @@ public:
 	InvalidFilenameException(const std::string &s): BaseException(s) {}
 };
 
-class ProcessingLimitException : public BaseException {
-public:
-	ProcessingLimitException(const std::string &s): BaseException(s) {}
-};
-
-class CommandLineError : public BaseException {
-public:
-	CommandLineError(const std::string &s): BaseException(s) {}
-};
-
 class ItemNotFoundException : public BaseException {
 public:
 	ItemNotFoundException(const std::string &s): BaseException(s) {}
@@ -159,22 +129,3 @@ public:
 		BaseException(s)
 	{}
 };
-
-class TargetInexistentException : public std::exception
-{
-	virtual const char * what() const throw()
-	{
-		return "Somebody tried to refer to something that doesn't exist.";
-	}
-};
-
-class NullPointerException : public std::exception
-{
-	virtual const char * what() const throw()
-	{
-		return "NullPointerException";
-	}
-};
-
-#endif
-
