@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef GAME_PARAMS_H
-#define GAME_PARAMS_H
+#pragma once
 
 #include "irrlichttypes.h"
 
@@ -31,5 +30,3 @@ struct GameParams
 	SubgameSpec game_spec;
 	bool is_dedicated_server;
 };
-
-#endif
