@@ -144,10 +144,10 @@ GUIEngine::GUIEngine(JoystickController *joystick,
 	//create soundmanager
 	MenuMusicFetcher soundfetcher;
 #if USE_SOUND
-	if (g_settings->getBool("enable_sound"))
+	if (g_settings->getBool("enable_sound") && g_sound_manager_singleton.get())
 		m_sound_manager = createOpenALSoundManager(g_sound_manager_singleton.get(), &soundfetcher);
 #endif
-	if(!m_sound_manager)
+	if (!m_sound_manager)
 		m_sound_manager = &dummySoundManager;
 
 	//create topleft header
