@@ -18,8 +18,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef ITEMDEF_HEADER
-#define ITEMDEF_HEADER
+#pragma once
 
 #include "irrlichttypes_extrabloated.h"
 #include <string>
@@ -27,6 +26,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <set>
 #include "itemgroup.h"
 #include "sound.h"
+#include "texture_override.h" // TextureOverride
 class IGameDef;
 class Client;
 struct ToolCapabilities;
@@ -61,7 +61,9 @@ struct ItemDefinition
 		Visual properties
 	*/
 	std::string inventory_image; // Optional for nodes, mandatory for tools/craftitems
+	std::string inventory_overlay; // Overlay of inventory_image.
 	std::string wield_image; // If empty, inventory_image or mesh (only nodes) is used
+	std::string wield_overlay; // Overlay of wield_image.
 	std::string palette_image; // If specified, the item will be colorized based on this
 	video::SColor color; // The fallback color of the node.
 	v3f wield_scale;
@@ -101,8 +103,9 @@ private:
 class IItemDefManager
 {
 public:
-	IItemDefManager(){}
-	virtual ~IItemDefManager(){}
+	IItemDefManager() = default;
+
+	virtual ~IItemDefManager() = default;
 
 	// Get item definition
 	virtual const ItemDefinition& get(const std::string &name) const=0;
@@ -134,8 +137,9 @@ public:
 class IWritableItemDefManager : public IItemDefManager
 {
 public:
-	IWritableItemDefManager(){}
-	virtual ~IWritableItemDefManager(){}
+	IWritableItemDefManager() = default;
+
+	virtual ~IWritableItemDefManager() = default;
 
 	// Get item definition
 	virtual const ItemDefinition& get(const std::string &name) const=0;
@@ -154,6 +158,10 @@ public:
 		Client *client) const=0;
 #endif
 
+	// Replace the textures of registered nodes with the ones specified in
+	// the texture pack's override.txt files
+	virtual void applyTextureOverrides(const std::vector<TextureOverride> &overrides)=0;
+
 	// Remove all registered item and node definitions and aliases
 	// Then re-add the builtin item definitions
 	virtual void clear()=0;
@@ -174,5 +182,3 @@ public:
 };
 
 IWritableItemDefManager* createItemDefManager();
-
-#endif
