@@ -31,6 +31,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "voxel.h"
 #include "modifiedstate.h"
 #include "util/container.h"
+#include "util/metricsbackend.h"
 #include "nodetimer.h"
 #include "map_settings_manager.h"
 #include "debug.h"
@@ -45,6 +46,7 @@ class NodeMetadata;
 class IGameDef;
 class IRollbackManager;
 class EmergeManager;
+class MetricsBackend;
 class ServerEnvironment;
 struct BlockMakeData;
 
@@ -324,7 +326,7 @@ public:
 	/*
 		savedir: directory to which map data should be saved
 	*/
-	ServerMap(const std::string &savedir, IGameDef *gamedef, EmergeManager *emerge);
+	ServerMap(const std::string &savedir, IGameDef *gamedef, EmergeManager *emerge, MetricsBackend *mb);
 	~ServerMap();
 
 	s32 mapType() const
@@ -380,21 +382,12 @@ public:
 		names when saving
 	*/
 	void createDirs(const std::string &path);
-	// returns something like "map/sectors/xxxxxxxx"
-	std::string getSectorDir(v2s16 pos, int layout = 2);
-	// dirname: final directory name
-	v2s16 getSectorPos(const std::string &dirname);
-	v3s16 getBlockPos(const std::string &sectordir, const std::string &blockfile);
-	static std::string getBlockFilename(v3s16 p);
 
 	/*
 		Database functions
 	*/
 	static MapDatabase *createDatabase(const std::string &name, const std::string &savedir, Settings &conf);
 
-	// Returns true if the database file does not exist
-	bool loadFromFolders();
-
 	// Call these before and after saving of blocks
 	void beginSave();
 	void endSave();
@@ -407,9 +400,6 @@ public:
 
 	bool saveBlock(MapBlock *block);
 	static bool saveBlock(MapBlock *block, MapDatabase *db);
-	// This will generate a sector with getSector if not found.
-	void loadBlock(const std::string &sectordir, const std::string &blockfile,
-			MapSector *sector, bool save_after_load=false);
 	MapBlock* loadBlock(v3s16 p);
 	// Database version
 	void loadBlock(std::string *blob, v3s16 p3d, MapSector *sector, bool save_after_load=false);
@@ -460,6 +450,8 @@ private:
 	bool m_map_metadata_changed = true;
 	MapDatabase *dbase = nullptr;
 	MapDatabase *dbase_ro = nullptr;
+
+	MetricCounterPtr m_save_time_counter;
 };
 
 
