@@ -37,15 +37,18 @@ struct MapgenCarpathianParams : public MapgenParams
 	float river_depth      = 24.0f;
 	float valley_width     = 0.25f;
 
-	u32 spflags            = MGCARPATHIAN_CAVERNS;
-	float cave_width       = 0.09f;
-	s16 large_cave_depth   = -33;
-	s16 lava_depth         = -256;
-	s16 cavern_limit       = -256;
-	s16 cavern_taper       = 256;
-	float cavern_threshold = 0.7f;
-	s16 dungeon_ymin       = -31000;
-	s16 dungeon_ymax       = 31000;
+	float cave_width         = 0.09f;
+	s16 large_cave_depth     = -33;
+	u16 small_cave_num_min   = 0;
+	u16 small_cave_num_max   = 0;
+	u16 large_cave_num_min   = 0;
+	u16 large_cave_num_max   = 2;
+	float large_cave_flooded = 0.5f;
+	s16 cavern_limit         = -256;
+	s16 cavern_taper         = 256;
+	float cavern_threshold   = 0.7f;
+	s16 dungeon_ymin         = -31000;
+	s16 dungeon_ymax         = 31000;
 
 	NoiseParams np_filler_depth;
 	NoiseParams np_height1;
@@ -70,12 +73,13 @@ struct MapgenCarpathianParams : public MapgenParams
 
 	void readParams(const Settings *settings);
 	void writeParams(Settings *settings) const;
+	void setDefaultSettings(Settings *settings);
 };
 
 class MapgenCarpathian : public MapgenBasic
 {
 public:
-	MapgenCarpathian(MapgenCarpathianParams *params, EmergeManager *emerge);
+	MapgenCarpathian(MapgenCarpathianParams *params, EmergeParams *emerge);
 	~MapgenCarpathian();
 
 	virtual MapgenType getType() const { return MAPGEN_CARPATHIAN; }
@@ -89,10 +93,6 @@ private:
 	float river_depth;
 	float valley_width;
 
-	s16 large_cave_depth;
-	s16 dungeon_ymin;
-	s16 dungeon_ymax;
-
 	Noise *noise_height1;
 	Noise *noise_height2;
 	Noise *noise_height3;
