@@ -1,7 +1,7 @@
 /*
 Minetest
-Copyright (C) 2013-2018 kwolekr, Ryan Kwolek <kwolekr@minetest.net>
-Copyright (C) 2014-2018 paramat
+Copyright (C) 2014-2020 paramat
+Copyright (C) 2013-2016 kwolekr, Ryan Kwolek <kwolekr@minetest.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -35,17 +35,21 @@ extern FlagDesc flagdesc_mapgen_v7[];
 
 
 struct MapgenV7Params : public MapgenParams {
-	u32 spflags = MGV7_MOUNTAINS | MGV7_RIDGES | MGV7_CAVERNS;
 	s16 mount_zero_level = 0;
-	float float_mount_density = 0.6f;
-	float float_mount_height = 128.0f;
-	float float_mount_exponent = 0.75f;
-	s16 floatland_level = 1280;
-	s16 shadow_limit = 1024;
+	s16 floatland_ymin = 1024;
+	s16 floatland_ymax = 4096;
+	s16 floatland_taper = 256;
+	float float_taper_exp = 2.0f;
+	float floatland_density = -0.6f;
+	s16 floatland_ywater = -31000;
 
 	float cave_width = 0.09f;
 	s16 large_cave_depth = -33;
-	s16 lava_depth = -256;
+	u16 small_cave_num_min = 0;
+	u16 small_cave_num_max = 0;
+	u16 large_cave_num_min = 0;
+	u16 large_cave_num_max = 2;
+	float large_cave_flooded = 0.5f;
 	s16 cavern_limit = -256;
 	s16 cavern_taper = 256;
 	float cavern_threshold = 0.7f;
@@ -59,10 +63,9 @@ struct MapgenV7Params : public MapgenParams {
 	NoiseParams np_filler_depth;
 	NoiseParams np_mount_height;
 	NoiseParams np_ridge_uwater;
-	NoiseParams np_floatland_base;
-	NoiseParams np_float_base_height;
 	NoiseParams np_mountain;
 	NoiseParams np_ridge;
+	NoiseParams np_floatland;
 	NoiseParams np_cavern;
 	NoiseParams np_cave1;
 	NoiseParams np_cave2;
@@ -73,12 +76,13 @@ struct MapgenV7Params : public MapgenParams {
 
 	void readParams(const Settings *settings);
 	void writeParams(Settings *settings) const;
+	void setDefaultSettings(Settings *settings);
 };
 
 
 class MapgenV7 : public MapgenBasic {
 public:
-	MapgenV7(MapgenV7Params *params, EmergeManager *emerge);
+	MapgenV7(MapgenV7Params *params, EmergeParams *emerge);
 	~MapgenV7();
 
 	virtual MapgenType getType() const { return MAPGEN_V7; }
@@ -90,23 +94,21 @@ public:
 	float baseTerrainLevelFromMap(int index);
 	bool getMountainTerrainAtPoint(s16 x, s16 y, s16 z);
 	bool getMountainTerrainFromMap(int idx_xyz, int idx_xz, s16 y);
-	bool getFloatlandMountainFromMap(int idx_xyz, int idx_xz, s16 y);
-	void floatBaseExtentFromMap(s16 *float_base_min, s16 *float_base_max, int idx_xz);
+	bool getFloatlandTerrainFromMap(int idx_xyz, float float_offset);
 
 	int generateTerrain();
 	void generateRidgeTerrain();
 
 private:
 	s16 mount_zero_level;
-	float float_mount_density;
-	float float_mount_height;
-	float float_mount_exponent;
-	s16 floatland_level;
-	s16 shadow_limit;
+	s16 floatland_ymin;
+	s16 floatland_ymax;
+	s16 floatland_taper;
+	float float_taper_exp;
+	float floatland_density;
+	s16 floatland_ywater;
 
-	s16 large_cave_depth;
-	s16 dungeon_ymin;
-	s16 dungeon_ymax;
+	float *float_offset_cache = nullptr;
 
 	Noise *noise_terrain_base;
 	Noise *noise_terrain_alt;
@@ -114,8 +116,7 @@ private:
 	Noise *noise_height_select;
 	Noise *noise_mount_height;
 	Noise *noise_ridge_uwater;
-	Noise *noise_floatland_base;
-	Noise *noise_float_base_height;
 	Noise *noise_mountain;
 	Noise *noise_ridge;
+	Noise *noise_floatland;
 };
