@@ -30,16 +30,15 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <string>
 
 namespace porting {
-/** java app **/
+// java app
 extern android_app *app_global;
 
-/** java <-> c++ interaction interface **/
+// java <-> c++ interaction interface
 extern JNIEnv *jnienv;
 
-/**
- * do initialization required on android only
- */
+// do initialization required on android only
 void initAndroid();
+
 void cleanupAndroid();
 
 /**
@@ -48,11 +47,6 @@ void cleanupAndroid();
  */
 void initializePathsAndroid();
 
-/**
- * use java function to copy media from assets to external storage
- */
-void copyAssets();
-
 /**
  * show text input dialog in java
  * @param acceptButton text to display on accept button
@@ -61,8 +55,10 @@ void copyAssets();
  * @param editType type of texfield
  * (1==multiline text input; 2==single line text input; 3=password field)
  */
-void showInputDialog(const std::string& acceptButton,
-		const  std::string& hint, const std::string& current, int editType);
+void showInputDialog(const std::string &acceptButton,
+					const std::string &hint, const std::string &current, int editType);
+
+void openURLAndroid(const std::string &url);
 
 /**
  * WORKAROUND for not working callbacks from java -> c++
@@ -80,5 +76,4 @@ std::string getInputDialogValue();
 float getDisplayDensity();
 v2u32 getDisplaySize();
 #endif
-
 }
