@@ -24,6 +24,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 struct ObjectProperties;
 struct ToolCapabilities;
+struct collisionMoveResult;
 
 class ScriptApiEntity
 		: virtual public ScriptApiBase
@@ -35,8 +36,9 @@ public:
 	void luaentity_Remove(u16 id);
 	std::string luaentity_GetStaticdata(u16 id);
 	void luaentity_GetProperties(u16 id,
-			ObjectProperties *prop);
-	void luaentity_Step(u16 id, float dtime);
+			ServerActiveObject *self, ObjectProperties *prop);
+	void luaentity_Step(u16 id, float dtime,
+		const collisionMoveResult *moveresult);
 	bool luaentity_Punch(u16 id,
 			ServerActiveObject *puncher, float time_from_last_punch,
 			const ToolCapabilities *toolcap, v3f dir, s16 damage);
