@@ -36,7 +36,7 @@ public:
 	virtual ~ScriptApiNode() = default;
 
 	bool node_on_punch(v3s16 p, MapNode node,
-			ServerActiveObject *puncher, PointedThing pointed);
+			ServerActiveObject *puncher, const PointedThing &pointed);
 	bool node_on_dig(v3s16 p, MapNode node,
 			ServerActiveObject *digger);
 	void node_on_construct(v3s16 p, MapNode node);
