@@ -32,26 +32,29 @@ extern "C" {
 
 #ifndef SERVER
 class Client;
+class GUIEngine;
 #endif
 
 class ScriptApiBase;
 class Server;
 class Environment;
-class GUIEngine;
+class ServerInventoryManager;
 
 class ModApiBase : protected LuaHelper {
 
 public:
 	static ScriptApiBase*   getScriptApiBase(lua_State *L);
 	static Server*          getServer(lua_State *L);
+	static ServerInventoryManager *getServerInventoryMgr(lua_State *L);
 	#ifndef SERVER
 	static Client*          getClient(lua_State *L);
+	static GUIEngine*       getGuiEngine(lua_State *L);
 	#endif // !SERVER
 
 	static IGameDef*        getGameDef(lua_State *L);
 
 	static Environment*     getEnv(lua_State *L);
-	static GUIEngine*       getGuiEngine(lua_State *L);
+
 	// When we are not loading the mod, this function returns "."
 	static std::string      getCurrentModPath(lua_State *L);
 
