@@ -32,12 +32,21 @@ private:
 	// garbage collector
 	static int gc_object(lua_State *L);
 
+	// get_velocity(self)
 	static int l_get_velocity(lua_State *L);
 
+	// get_hp(self)
 	static int l_get_hp(lua_State *L);
 
+	// get_name(self)
 	static int l_get_name(lua_State *L);
 
+	// get_wield_index(self)
+	static int l_get_wield_index(lua_State *L);
+
+	// get_wielded_item(self)
+	static int l_get_wielded_item(lua_State *L);
+
 	static int l_is_attached(lua_State *L);
 	static int l_is_touching_ground(lua_State *L);
 	static int l_is_in_liquid(lua_State *L);
@@ -54,18 +63,28 @@ private:
 	static int l_get_last_velocity(lua_State *L);
 	static int l_get_last_look_vertical(lua_State *L);
 	static int l_get_last_look_horizontal(lua_State *L);
-	static int l_get_key_pressed(lua_State *L);
 
+	// get_control(self)
+	static int l_get_control(lua_State *L);
+
+	// get_breath(self)
 	static int l_get_breath(lua_State *L);
 
+	// get_pos(self)
 	static int l_get_pos(lua_State *L);
 
+	// get_movement_acceleration(self)
 	static int l_get_movement_acceleration(lua_State *L);
 
+	// get_movement_speed(self)
 	static int l_get_movement_speed(lua_State *L);
 
+	// get_movement(self)
 	static int l_get_movement(lua_State *L);
 
+	// get_armor_groups(self)
+	static int l_get_armor_groups(lua_State *L);
+
 	// hud_add(self, id, form)
 	static int l_hud_add(lua_State *L);
 
