@@ -1,6 +1,7 @@
 /*
 Minetest
 Copyright (C) 2013 celeron55, Perttu Ahola <celeron55@gmail.com>
+Copyright (C) 2017-8 rubenwardy <rw@rubenwardy.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -50,6 +51,42 @@ MetaDataRef* MetaDataRef::checkobject(lua_State *L, int narg)
 
 // Exported functions
 
+// contains(self, name)
+int MetaDataRef::l_contains(lua_State *L)
+{
+	MAP_LOCK_REQUIRED;
+
+	MetaDataRef *ref = checkobject(L, 1);
+	std::string name = luaL_checkstring(L, 2);
+
+	Metadata *meta = ref->getmeta(false);
+	if (meta == NULL)
+		return 0;
+
+	lua_pushboolean(L, meta->contains(name));
+	return 1;
+}
+
+// get(self, name)
+int MetaDataRef::l_get(lua_State *L)
+{
+	MAP_LOCK_REQUIRED;
+
+	MetaDataRef *ref = checkobject(L, 1);
+	std::string name = luaL_checkstring(L, 2);
+
+	Metadata *meta = ref->getmeta(false);
+	if (meta == NULL)
+		return 0;
+
+	std::string str;
+	if (meta->getStringToRef(name, str)) {
+		lua_pushlstring(L, str.c_str(), str.size());
+		return 1;
+	}
+	return 0;
+}
+
 // get_string(self, name)
 int MetaDataRef::l_get_string(lua_State *L)
 {
@@ -85,7 +122,7 @@ int MetaDataRef::l_set_string(lua_State *L)
 		return 0;
 
 	meta->setString(name, str);
-	ref->reportMetadataChange();
+	ref->reportMetadataChange(&name);
 	return 0;
 }
 
@@ -116,14 +153,16 @@ int MetaDataRef::l_set_int(lua_State *L)
 	MetaDataRef *ref = checkobject(L, 1);
 	std::string name = luaL_checkstring(L, 2);
 	int a = luaL_checkint(L, 3);
-	std::string str = itos(a);
+	std::string str;
+	if (a != 0)
+		str = itos(a);
 
 	Metadata *meta = ref->getmeta(true);
 	if (meta == NULL || str == meta->getString(name))
 		return 0;
 
 	meta->setString(name, str);
-	ref->reportMetadataChange();
+	ref->reportMetadataChange(&name);
 	return 0;
 }
 
@@ -153,15 +192,17 @@ int MetaDataRef::l_set_float(lua_State *L)
 
 	MetaDataRef *ref = checkobject(L, 1);
 	std::string name = luaL_checkstring(L, 2);
-	float a = luaL_checknumber(L, 3);
-	std::string str = ftos(a);
+	float a = readParam<float>(L, 3);
+	std::string str;
+	if (a != 0)
+		str = ftos(a);
 
 	Metadata *meta = ref->getmeta(true);
 	if (meta == NULL || str == meta->getString(name))
 		return 0;
 
 	meta->setString(name, str);
-	ref->reportMetadataChange();
+	ref->reportMetadataChange(&name);
 	return 0;
 }
 
@@ -218,10 +259,9 @@ void MetaDataRef::handleToTable(lua_State *L, Metadata *meta)
 	lua_newtable(L);
 	{
 		const StringMap &fields = meta->getStrings();
-		for (StringMap::const_iterator
-				it = fields.begin(); it != fields.end(); ++it) {
-			const std::string &name = it->first;
-			const std::string &value = it->second;
+		for (const auto &field : fields) {
+			const std::string &name = field.first;
+			const std::string &value = field.second;
 			lua_pushlstring(L, name.c_str(), name.size());
 			lua_pushlstring(L, value.c_str(), value.size());
 			lua_settable(L, -3);
@@ -239,7 +279,7 @@ bool MetaDataRef::handleFromTable(lua_State *L, int table, Metadata *meta)
 		lua_pushnil(L);
 		while (lua_next(L, fieldstable) != 0) {
 			// key at index -2 and value at index -1
-			std::string name = lua_tostring(L, -2);
+			std::string name = readParam<std::string>(L, -2);
 			size_t cl;
 			const char *cs = lua_tolstring(L, -1, &cl);
 			meta->setString(name, std::string(cs, cl));
