@@ -22,6 +22,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "common/c_converter.h"
 #include "common/c_content.h"
 #include "cpp_api/s_base.h"
+#include "cpp_api/s_security.h"
 #include "server.h"
 #include "environment.h"
 #include "remoteplayer.h"
@@ -138,52 +139,54 @@ int ModApiServer::l_get_player_ip(lua_State *L)
 // get_player_information(name)
 int ModApiServer::l_get_player_information(lua_State *L)
 {
-
 	NO_MAP_LOCK_REQUIRED;
-	const char * name = luaL_checkstring(L, 1);
-	RemotePlayer *player = dynamic_cast<ServerEnvironment *>(getEnv(L))->getPlayer(name);
-	if (player == NULL) {
+
+	Server *server = getServer(L);
+
+	const char *name = luaL_checkstring(L, 1);
+	RemotePlayer *player = server->getEnv().getPlayer(name);
+	if (!player) {
 		lua_pushnil(L); // no such player
 		return 1;
 	}
 
 	Address addr;
-	try
-	{
-		addr = getServer(L)->getPeerAddress(player->getPeerId());
-	} catch(const con::PeerNotFoundException &) {
+	try {
+		addr = server->getPeerAddress(player->getPeerId());
+	} catch (const con::PeerNotFoundException &) {
 		dstream << FUNCTION_NAME << ": peer was not found" << std::endl;
 		lua_pushnil(L); // error
 		return 1;
 	}
 
-	float min_rtt,max_rtt,avg_rtt,min_jitter,max_jitter,avg_jitter;
+	float min_rtt, max_rtt, avg_rtt, min_jitter, max_jitter, avg_jitter;
 	ClientState state;
 	u32 uptime;
 	u16 prot_vers;
-	u8 ser_vers,major,minor,patch;
-	std::string vers_string;
+	u8 ser_vers, major, minor, patch;
+	std::string vers_string, lang_code;
 
-#define ERET(code)                                                             \
-	if (!(code)) {                                                             \
-		dstream << FUNCTION_NAME << ": peer was not found" << std::endl;     \
-		lua_pushnil(L); /* error */                                            \
-		return 1;                                                              \
+	auto getConInfo = [&] (con::rtt_stat_type type, float *value) -> bool {
+		return server->getClientConInfo(player->getPeerId(), type, value);
+	};
+
+	bool have_con_info =
+		getConInfo(con::MIN_RTT, &min_rtt) &&
+		getConInfo(con::MAX_RTT, &max_rtt) &&
+		getConInfo(con::AVG_RTT, &avg_rtt) &&
+		getConInfo(con::MIN_JITTER, &min_jitter) &&
+		getConInfo(con::MAX_JITTER, &max_jitter) &&
+		getConInfo(con::AVG_JITTER, &avg_jitter);
+
+	bool r = server->getClientInfo(player->getPeerId(), &state, &uptime,
+		&ser_vers, &prot_vers, &major, &minor, &patch, &vers_string,
+		&lang_code);
+	if (!r) {
+		dstream << FUNCTION_NAME << ": peer was not found" << std::endl;
+		lua_pushnil(L); // error
+		return 1;
 	}
 
-	ERET(getServer(L)->getClientConInfo(player->getPeerId(), con::MIN_RTT, &min_rtt))
-	ERET(getServer(L)->getClientConInfo(player->getPeerId(), con::MAX_RTT, &max_rtt))
-	ERET(getServer(L)->getClientConInfo(player->getPeerId(), con::AVG_RTT, &avg_rtt))
-	ERET(getServer(L)->getClientConInfo(player->getPeerId(), con::MIN_JITTER,
-		&min_jitter))
-	ERET(getServer(L)->getClientConInfo(player->getPeerId(), con::MAX_JITTER,
-		&max_jitter))
-	ERET(getServer(L)->getClientConInfo(player->getPeerId(), con::AVG_JITTER,
-		&avg_jitter))
-
-	ERET(getServer(L)->getClientInfo(player->getPeerId(), &state, &uptime, &ser_vers,
-		&prot_vers, &major, &minor, &patch, &vers_string))
-
 	lua_newtable(L);
 	int table = lua_gettop(L);
 
@@ -201,29 +204,31 @@ int ModApiServer::l_get_player_information(lua_State *L)
 	}
 	lua_settable(L, table);
 
-	lua_pushstring(L,"min_rtt");
-	lua_pushnumber(L, min_rtt);
-	lua_settable(L, table);
+	if (have_con_info) { // may be missing
+		lua_pushstring(L, "min_rtt");
+		lua_pushnumber(L, min_rtt);
+		lua_settable(L, table);
 
-	lua_pushstring(L,"max_rtt");
-	lua_pushnumber(L, max_rtt);
-	lua_settable(L, table);
+		lua_pushstring(L, "max_rtt");
+		lua_pushnumber(L, max_rtt);
+		lua_settable(L, table);
 
-	lua_pushstring(L,"avg_rtt");
-	lua_pushnumber(L, avg_rtt);
-	lua_settable(L, table);
+		lua_pushstring(L, "avg_rtt");
+		lua_pushnumber(L, avg_rtt);
+		lua_settable(L, table);
 
-	lua_pushstring(L,"min_jitter");
-	lua_pushnumber(L, min_jitter);
-	lua_settable(L, table);
+		lua_pushstring(L, "min_jitter");
+		lua_pushnumber(L, min_jitter);
+		lua_settable(L, table);
 
-	lua_pushstring(L,"max_jitter");
-	lua_pushnumber(L, max_jitter);
-	lua_settable(L, table);
+		lua_pushstring(L, "max_jitter");
+		lua_pushnumber(L, max_jitter);
+		lua_settable(L, table);
 
-	lua_pushstring(L,"avg_jitter");
-	lua_pushnumber(L, avg_jitter);
-	lua_settable(L, table);
+		lua_pushstring(L, "avg_jitter");
+		lua_pushnumber(L, avg_jitter);
+		lua_settable(L, table);
+	}
 
 	lua_pushstring(L,"connection_uptime");
 	lua_pushnumber(L, uptime);
@@ -237,6 +242,10 @@ int ModApiServer::l_get_player_information(lua_State *L)
 	lua_pushnumber(L, player->formspec_version);
 	lua_settable(L, table);
 
+	lua_pushstring(L, "lang_code");
+	lua_pushstring(L, lang_code.c_str());
+	lua_settable(L, table);
+
 #ifndef NDEBUG
 	lua_pushstring(L,"serialization_version");
 	lua_pushnumber(L, ser_vers);
@@ -263,7 +272,6 @@ int ModApiServer::l_get_player_information(lua_State *L)
 	lua_settable(L, table);
 #endif
 
-#undef ERET
 	return 1;
 }
 
@@ -405,9 +413,6 @@ int ModApiServer::l_get_modnames(lua_State *L)
 	std::vector<std::string> modlist;
 	getServer(L)->getModNames(modlist);
 
-	// Take unsorted items from mods_unsorted and sort them into
-	// mods_sorted; not great performance but the number of mods on a
-	// server will likely be small.
 	std::sort(modlist.begin(), modlist.end());
 
 	// Package them up for Lua
@@ -467,6 +472,23 @@ int ModApiServer::l_sound_fade(lua_State *L)
 	return 0;
 }
 
+// dynamic_add_media(filepath)
+int ModApiServer::l_dynamic_add_media(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	// Reject adding media before the server has started up
+	if (!getEnv(L))
+		throw LuaError("Dynamic media cannot be added before server has started up");
+
+	std::string filepath = readParam<std::string>(L, 1);
+	CHECK_SECURE_PATH(L, filepath.c_str(), false);
+
+	bool ok = getServer(L)->dynamicAddMedia(filepath);
+	lua_pushboolean(L, ok);
+	return 1;
+}
+
 // is_singleplayer()
 int ModApiServer::l_is_singleplayer(lua_State *L)
 {
@@ -531,6 +553,7 @@ void ModApiServer::Initialize(lua_State *L, int top)
 	API_FCT(sound_play);
 	API_FCT(sound_stop);
 	API_FCT(sound_fade);
+	API_FCT(dynamic_add_media);
 
 	API_FCT(get_player_information);
 	API_FCT(get_player_privs);
