@@ -19,11 +19,10 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include <iostream>
 #include "config.h"
-#include "mods.h"
+#include "content/mods.h"
 #include <json/json.h>
 
-#ifndef SERVERLIST_HEADER
-#define SERVERLIST_HEADER
+#pragma once
 
 typedef Json::Value ServerListSpec;
 
@@ -51,5 +50,3 @@ void sendAnnounce(AnnounceAction, u16 port,
 #endif
 
 } // namespace ServerList
-
-#endif
