@@ -28,35 +28,10 @@ DEALINGS IN THE SOFTWARE.
 #include "log.h"
 #include "porting.h"
 
-#define UNUSED(expr) do { (void)(expr); } while (0)
-
-#if USE_CPP11_THREADS
-	#include <chrono>
-	#include <system_error>
-#elif USE_WIN_THREADS
-	#ifndef _WIN32_WCE
-		#include <process.h>
-	#endif
-#elif USE_POSIX_THREADS
-	#include <time.h>
-	#include <assert.h>
-	#include <stdlib.h>
-	#include <unistd.h>
-	#include <sys/time.h>
-
-	#if defined(__FreeBSD__) || defined(__APPLE__)
-		#include <sys/types.h>
-		#include <sys/sysctl.h>
-	#elif defined(_GNU_SOURCE)
-		#include <sys/sysinfo.h>
-	#endif
-#endif
-
-
 // for setName
 #if defined(__linux__)
 	#include <sys/prctl.h>
-#elif defined(__FreeBSD__) || defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 	#include <pthread_np.h>
 #elif defined(_MSC_VER)
 	struct THREADNAME_INFO {
@@ -70,8 +45,6 @@ DEALINGS IN THE SOFTWARE.
 // for bindToProcessor
 #if __FreeBSD_version >= 702106
 	typedef cpuset_t cpu_set_t;
-#elif defined(__linux__)
-	#include <sched.h>
 #elif defined(__sun) || defined(sun)
 	#include <sys/types.h>
 	#include <sys/processor.h>
@@ -87,8 +60,6 @@ DEALINGS IN THE SOFTWARE.
 
 Thread::Thread(const std::string &name) :
 	m_name(name),
-	m_retval(NULL),
-	m_joinable(false),
 	m_request_stop(false),
 	m_running(false)
 {
@@ -121,28 +92,12 @@ bool Thread::start()
 	// The mutex may already be locked if the thread is being restarted
 	m_start_finished_mutex.try_lock();
 
-#if USE_CPP11_THREADS
-
 	try {
 		m_thread_obj = new std::thread(threadProc, this);
 	} catch (const std::system_error &e) {
 		return false;
 	}
 
-#elif USE_WIN_THREADS
-
-	m_thread_handle = CreateThread(NULL, 0, threadProc, this, 0, &m_thread_id);
-	if (!m_thread_handle)
-		return false;
-
-#elif USE_POSIX_THREADS
-
-	int status = pthread_create(&m_thread_handle, NULL, threadProc, this);
-	if (status)
-		return false;
-
-#endif
-
 	// Allow spawned thread to continue
 	m_start_finished_mutex.unlock();
 
@@ -169,30 +124,11 @@ bool Thread::wait()
 	if (!m_joinable)
 		return false;
 
-#if USE_CPP11_THREADS
 
 	m_thread_obj->join();
 
 	delete m_thread_obj;
-	m_thread_obj = NULL;
-
-#elif USE_WIN_THREADS
-
-	int ret = WaitForSingleObject(m_thread_handle, INFINITE);
-	assert(ret == WAIT_OBJECT_0);
-	UNUSED(ret);
-
-	CloseHandle(m_thread_handle);
-	m_thread_handle = NULL;
-	m_thread_id = -1;
-
-#elif USE_POSIX_THREADS
-
-	int ret = pthread_join(m_thread_handle, NULL);
-	assert(ret == 0);
-	UNUSED(ret);
-
-#endif
+	m_thread_obj = nullptr;
 
 	assert(m_running == false);
 	m_joinable = false;
@@ -209,9 +145,10 @@ bool Thread::kill()
 
 	m_running = false;
 
-#if USE_WIN_THREADS
-	TerminateThread(m_thread_handle, 0);
-	CloseHandle(m_thread_handle);
+#if defined(_WIN32)
+	// See https://msdn.microsoft.com/en-us/library/hh920601.aspx#thread__native_handle_method
+	TerminateThread((HANDLE) m_thread_obj->native_handle(), 0);
+	CloseHandle((HANDLE) m_thread_obj->native_handle());
 #else
 	// We need to pthread_kill instead on Android since NDKv5's pthread
 	// implementation is incomplete.
@@ -223,7 +160,7 @@ bool Thread::kill()
 	wait();
 #endif
 
-	m_retval       = NULL;
+	m_retval       = nullptr;
 	m_joinable     = false;
 	m_request_stop = false;
 
@@ -241,16 +178,8 @@ bool Thread::getReturnValue(void **ret)
 }
 
 
-#if USE_CPP11_THREADS || USE_POSIX_THREADS
-void *Thread::threadProc(void *param)
-#elif defined(_WIN32_WCE)
-DWORD Thread::threadProc(LPVOID param)
-#elif defined(_WIN32)
-DWORD WINAPI Thread::threadProc(LPVOID param)
-#endif
+void Thread::threadProc(Thread *thr)
 {
-	Thread *thr = (Thread *)param;
-
 #ifdef _AIX
 	thr->m_kernel_thread_id = thread_self();
 #endif
@@ -272,9 +201,6 @@ DWORD WINAPI Thread::threadProc(LPVOID param)
 	// released. We try to unlock it from caller thread and it's refused by system.
 	thr->m_start_finished_mutex.unlock();
 	g_logger.deregisterThread();
-
-	// 0 is returned here to avoid an unnecessary ifdef clause
-	return 0;
 }
 
 
@@ -287,7 +213,7 @@ void Thread::setName(const std::string &name)
 	// distributions are still runing 2.11 and previous versions.
 	prctl(PR_SET_NAME, name.c_str());
 
-#elif defined(__FreeBSD__) || defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 
 	pthread_set_name_np(pthread_self(), name.c_str());
 
@@ -329,46 +255,7 @@ void Thread::setName(const std::string &name)
 
 unsigned int Thread::getNumberOfProcessors()
 {
-#if USE_CPP11_THREADS
-
 	return std::thread::hardware_concurrency();
-
-#elif USE_WIN_THREADS
-
-	SYSTEM_INFO sysinfo;
-	GetSystemInfo(&sysinfo);
-	return sysinfo.dwNumberOfProcessors;
-
-#elif defined(_SC_NPROCESSORS_ONLN)
-
-	return sysconf(_SC_NPROCESSORS_ONLN);
-
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || \
-	defined(__DragonFly__) || defined(__APPLE__)
-
-	unsigned int num_cpus = 1;
-	size_t len = sizeof(num_cpus);
-
-	int mib[2];
-	mib[0] = CTL_HW;
-	mib[1] = HW_NCPU;
-
-	sysctl(mib, 2, &num_cpus, &len, NULL, 0);
-	return num_cpus;
-
-#elif defined(_GNU_SOURCE)
-
-	return get_nprocs();
-
-#elif defined(PTW32_VERSION) || defined(__hpux)
-
-	return pthread_num_processors_np();
-
-#else
-
-	return 1;
-
-#endif
 }
 
 
@@ -378,11 +265,15 @@ bool Thread::bindToProcessor(unsigned int proc_number)
 
 	return false;
 
-#elif USE_WIN_THREADS
+#elif _MSC_VER
 
 	return SetThreadAffinityMask(getThreadHandle(), 1 << proc_number);
 
-#elif __FreeBSD_version >= 702106 || defined(__linux__)
+#elif __MINGW32__
+
+	return SetThreadAffinityMask(pthread_gethandle(getThreadHandle()), 1 << proc_number);
+
+#elif __FreeBSD_version >= 702106 || defined(__linux__) || defined(__DragonFly__)
 
 	cpu_set_t cpuset;
 
@@ -426,10 +317,14 @@ bool Thread::bindToProcessor(unsigned int proc_number)
 
 bool Thread::setPriority(int prio)
 {
-#if USE_WIN_THREADS
+#ifdef _MSC_VER
 
 	return SetThreadPriority(getThreadHandle(), prio);
 
+#elif __MINGW32__
+
+	return SetThreadPriority(pthread_gethandle(getThreadHandle()), prio);
+
 #else
 
 	struct sched_param sparam;
